# -*- encoding: utf-8 -*-
#= mdk_config.rb - MDK Library 設定ファイルクラス
#
#Copyright:: VeriTrans Inc.
#Authors::   VeriTrans Inc.
#

require 'tg_mdk/mdk_config/const'
require 'tg_mdk/mdk_config/validate'

module Veritrans
  module Tercerog
    module Mdk

      #
      # MdkConfig クラス
      #    各設定ファイルから値を取得するインターフェイスを提供します。
      #
      class MdkConfig

        #
        # ===MdkConfigのインスタンスを取得します。
        # @param:: user_properties ユーザ定義のプロパティファイル（可変長指定）
        # @return:: インスタンス
        #
        def self.instance(*user_properties)
          Thread.current[:tg_mdk_config_thread_data_key] ||= new(*user_properties)
        end

        #
        # ===コンストラクタ（private）
        # @param:: user_properties ユーザ定義のプロパティファイル（可変長指定）
        #
        def initialize(*user_properties)
          tg_mdk_directory = File.expand_path(File.dirname(__FILE__))
          mdk_root_directory = "#{tg_mdk_directory}/../.."

          @errors ||= []
          @properties = {}
          @propertie_files = [mdk_root_directory + "/tg_mdk.ini",
                              tg_mdk_directory + "/internal.ini",
                              tg_mdk_directory + "/mdk_dto/dto.ini"]
          @propertie_files.concat(user_properties) unless user_properties.empty?

          build_properties
          define_propertie_methods
        end
        private_class_method :new

        #
        # ===プロパティファイルを読み取り、インスタンス変数[Hash]にセットします。
        #
        def build_properties
          @propertie_files.each do |propertie_file|
            unless File.readable?(propertie_file)
              raise Veritrans::Tercerog::Mdk::MdkError.new(
                            Veritrans::Tercerog::Mdk::MdkMessage::MB01_NOT_FOUND_CONFIG_FILE, propertie_file)
            end
            open(propertie_file, 'r:utf-8').read.scan(/^\w.*/) do |line|
              key, value = line.split('=')
              if value.nil?
                @properties[key.strip.intern] = "" if key
              else
                @properties[key.strip.intern] = value.strip if key
              end
            end
          end
        rescue => e
          raise e
        end
        private :build_properties

        #
        # ===各パラメータへプロパティとしてアクセスするメソッドを定義します。
        #
        def define_propertie_methods
          Veritrans::Tercerog::Mdk::MdkConfig.constants.each do |const|
            Veritrans::Tercerog::Mdk::MdkConfig.class_eval do
              define_method const.downcase.intern do
                @properties[const.intern]
              end
            end
          end
        end
        private :define_propertie_methods

        #
        # ===各パラメータへHashとしてアクセスするメソッドを定義します。
        # READ_TIMEOUTの値取得時に、値がない場合はCONNECTION_TIMEOUTの値を返す
        # @param:: key キー
        #
        def [](key)
          return nil if key.nil?
          if key == :READ_TIMEOUT
            read_timeout = @properties[key]
            unless read_timeout.nil?
              read_timeout
            else
              @properties[:CONNECTION_TIMEOUT]
            end
          elsif key.kind_of? Symbol
            @properties[key]
          elsif key.kind_of? String
            @properties[key.intern]
          else
            nil
          end
        end

        #
        # ===キーに対応する値を上書き設定します
        # @param:: key キー
        # @param:: value キーに紐付く値
        #
        def overwrite(key, value)
          @properties[key.strip.intern] = value.strip
        end

        #
        # ===電文のヘッダ情報に載せるuser-agent情報の文字列を返す。
        #
        def get_user_agent()
          mdk_ver = self["MDK_VERSION"]
          mdk_dto_ver = self["MDK_DTO_VERSION"]
          ruby_ver = RUBY_VERSION
          return "VeriTrans 4GMDK/#{mdk_ver}/#{mdk_dto_ver} (Ruby #{ruby_ver})"
        end

      end
    end
  end
end
